package models

import (
	"eta/eta_index_lib/global"
	"eta/eta_index_lib/utils"
	"fmt"
	"gorm.io/gorm"
	"strconv"
	"strings"
	"time"
)

type BaseFromCoalmineIndex struct {
	BaseFromCoalmineCompanyIndexId int
	IndexName                      string `description:"持买单量指标名称"`
	IndexCode                      string `description:"持买单量指标编码"`
	DealValue                      string `description:"成交量"`
	DataTime                       string `description:"数据日期"`
	Source                         string `description:"来源"`
	Province                       string `description:"省份"`
	City                           string `description:"城市"`
	GroupName                      string `description:"集团名称"`
	Unit                           string `description:"单位"`
	Frequency                      string `description:"频率"`
	CreateTime                     string `description:"插入时间"`
	ModifyTime                     string `description:"修改时间"`
}

// AfterFind 在该模型上设置钩子函数，把日期转成正确的string，所以查询函数只能用Find函数，First或者Scan是不会触发该函数的来获取数据
func (m *BaseFromCoalmineIndex) AfterFind(db *gorm.DB) (err error) {
	m.DataTime = utils.GormDateStrToDateStr(m.DataTime)
	m.CreateTime = utils.GormDateStrToDateTimeStr(m.CreateTime)
	m.ModifyTime = utils.GormDateStrToDateTimeStr(m.ModifyTime)

	return
}

func GetBaseFromCoalIndexByCode(suffix, indexCode string) (items []*BaseFromCoalmineIndex, err error) {
	//o := orm.NewOrm()
	sql := `SELECT * FROM base_from_coalmine_%s WHERE index_code=? `
	sql = fmt.Sprintf(sql, suffix)
	//_, err = o.Raw(sql, indexCode).QueryRows(&items)
	err = global.DEFAULT_DB.Raw(sql, indexCode).Find(&items).Error
	return
}

func GetCoalDataByTradeCode(suffix, condition string, pars []interface{}) (item []*BaseFromCoalmineIndex, err error) {
	sql := ` SELECT * FROM base_from_coalmine_%s WHERE 1=1 `
	sql = fmt.Sprintf(sql, suffix)
	//o := orm.NewOrm()
	if condition != "" {
		sql += condition
	}
	sql += ` ORDER BY data_time DESC `
	err = global.DEFAULT_DB.Raw(sql, pars...).Find(&item).Error
	//_, err = o.Raw(sql, pars).QueryRows(&item)
	return
}

func AddEdbDataFromCoal(edbCode string) (err error) {
	var suffix string
	if strings.Contains(edbCode, "jsm") {
		suffix = "jsm_index"
	} else if strings.Contains(edbCode, "company") {
		suffix = "company_index"
	} else if strings.Contains(edbCode, "firm") {
		suffix = "firm_index"
	} else if strings.Contains(edbCode, "coastal") {
		suffix = "coastal_index"
	} else if strings.Contains(edbCode, "inland") {
		suffix = "inland_index"
	}

	//o := orm.NewOrm()

	coalBaseDataAll, err := GetBaseFromCoalIndexByCode(suffix, edbCode)
	if err != nil && !utils.IsErrNoRow(err) {
		return
	}

	var isAdd bool
	addSql := ` INSERT INTO edb_data_coal(edb_info_id,edb_code,data_time,value,create_time,modify_time,data_timestamp) values `
	addSql = utils.ReplaceDriverKeywords("", addSql)
	existMap := make(map[string]string)
	for _, sv := range coalBaseDataAll {
		eDate := sv.DataTime
		var timeStr string
		var dataTime time.Time
		var sDataTime string
		var timestamp int64
		if suffix == "firm_index" {
			syear := eDate[:4]
			year, _ := strconv.Atoi(syear)
			smonth := eDate[7:]
			smonth = strings.Split(smonth, "月份")[0]
			month, _ := strconv.Atoi(smonth)
			var day int
			if strings.Contains(eDate, "上旬") {
				day = 15
				currentLocation := time.Now().Location()
				dataTime = time.Date(year, time.Month(month), day, 0, 0, 0, 0, currentLocation)
				sDataTime = dataTime.Format(utils.FormatDate)
				timestamp = dataTime.UnixNano() / 1e6
				timeStr = fmt.Sprintf("%d", timestamp)
			} else if strings.Contains(eDate, "中旬") {
				day = 25
				currentLocation := time.Now().Location()
				dataTime = time.Date(year, time.Month(month), day, 0, 0, 0, 0, currentLocation)
				sDataTime = dataTime.Format(utils.FormatDate)
				timestamp = dataTime.UnixNano() / 1e6
				timeStr = fmt.Sprintf("%d", timestamp)
			} else {
				currentLocation := time.Now().Location()
				firstOfMonth := time.Date(year, time.Month(month), 1, 0, 0, 0, 0, currentLocation)
				dataTime = firstOfMonth.AddDate(0, 1, -1)
				sDataTime = dataTime.Format(utils.FormatDate)
				timestamp = dataTime.UnixNano() / 1e6
				timeStr = fmt.Sprintf("%d", timestamp)
			}
		} else {
			sDataTime = eDate
			dataTime, err = time.ParseInLocation(utils.FormatDate, eDate, time.Local)
			if err != nil {
				fmt.Println("time.Parse Err:" + eDate)
				return err
			}
			timestamp = dataTime.UnixNano() / 1e6
			timeStr = fmt.Sprintf("%d", timestamp)
		}

		value := strings.Replace(sv.DealValue, "%", "", -1)
		if _, ok := existMap[sDataTime]; !ok {
			addSql += GetAddSql("0", edbCode, sDataTime, timeStr, value)
			fmt.Println("edbCode：", edbCode)
			fmt.Println("sDataTime：", sDataTime)
			fmt.Println("timeStr：", timeStr)
			fmt.Println("value：", value)
			isAdd = true
		}
		existMap[eDate] = value
	}
	if isAdd {
		addSql = strings.TrimRight(addSql, ",")
		utils.FileLog.Info("addSql:" + addSql)
		//_, err = o.Raw(addSql).Exec()
		err = global.DEFAULT_DB.Exec(addSql).Error
		if err != nil {
			return err
		}
	}
	return
}

func RefreshEdbDataFromCoal(edbInfoId int, edbCode, startDate string) (err error) {
	source := utils.DATA_SOURCE_COAL
	subSource := utils.DATA_SUB_SOURCE_EDB
	//o := orm.NewOrm()
	if err != nil {
		return
	}
	var suffix string
	if strings.Contains(edbCode, "jsm") {
		suffix = "jsm_index"
	} else if strings.Contains(edbCode, "company") {
		suffix = "company_index"
	} else if strings.Contains(edbCode, "firm") {
		suffix = "firm_index"
	} else if strings.Contains(edbCode, "coastal") {
		suffix = "coastal_index"
	} else if strings.Contains(edbCode, "inland") {
		suffix = "inland_index"
	}
	edbInfoIdStr := strconv.Itoa(edbInfoId)
	//计算数据
	var condition string
	var pars []interface{}

	if edbCode != "" {
		condition += " AND index_code=? "
		pars = append(pars, edbCode)
	}

	if startDate != "" {
		condition += " AND data_time>=? "
		pars = append(pars, startDate)
	}

	glDataList, err := GetCoalDataByTradeCode(suffix, condition, pars)
	if err != nil {
		return
	}

	// 真实数据的最大日期  ， 插入规则配置的日期
	var realDataMaxDate, edbDataInsertConfigDate time.Time
	var edbDataInsertConfig *EdbDataInsertConfig
	var isFindConfigDateRealData bool //是否找到配置日期的实际数据的值
	{
		edbDataInsertConfig, err = GetEdbDataInsertConfigByEdbId(edbInfoId)
		if err != nil && !utils.IsErrNoRow(err) {
			return
		}
		if edbDataInsertConfig != nil {
			edbDataInsertConfigDate = edbDataInsertConfig.Date
		}
	}

	//获取指标所有数据
	var existCondition string
	var existPars []interface{}

	existCondition += " AND edb_info_id=? "
	existPars = append(existPars, edbInfoId)
	if startDate != "" {
		existCondition += " AND data_time>=? "
		existPars = append(existPars, startDate)
	}

	existList, err := GetEdbDataByCondition(source, subSource, existCondition, existPars)
	if err != nil {
		return err
	}
	existMap := make(map[string]*EdbInfoSearchData)
	for _, v := range existList {
		existMap[v.DataTime] = v
	}

	addSql := ` INSERT INTO edb_data_coal(edb_info_id,edb_code,data_time,value,create_time,modify_time,data_timestamp) values `
	addSql = utils.ReplaceDriverKeywords("", addSql)
	var isAdd bool
	for _, v := range glDataList {
		var value string
		value = strings.Replace(v.DealValue, "%", "", -1)
		item := v
		itemValue := value
		if _, ok := existMap[v.DataTime]; !ok {
			eDate := item.DataTime
			var timeStr string
			var dataTime time.Time
			var sDataTime string
			var timestamp int64
			if suffix == "firm_index" {
				syear := eDate[:4]
				year, _ := strconv.Atoi(syear)
				smonth := eDate[7:]
				smonth = strings.Split(smonth, "月份")[0]
				month, _ := strconv.Atoi(smonth)
				var day int
				if strings.Contains(eDate, "上旬") {
					day = 15
					currentLocation := time.Now().Location()
					dataTime = time.Date(year, time.Month(month), day, 0, 0, 0, 0, currentLocation)
					sDataTime = dataTime.Format(utils.FormatDate)
					timestamp = dataTime.UnixNano() / 1e6
					timeStr = fmt.Sprintf("%d", timestamp)
				} else if strings.Contains(eDate, "中旬") {
					day = 25
					currentLocation := time.Now().Location()
					dataTime = time.Date(year, time.Month(month), day, 0, 0, 0, 0, currentLocation)
					sDataTime = dataTime.Format(utils.FormatDate)
					timestamp = dataTime.UnixNano() / 1e6
					timeStr = fmt.Sprintf("%d", timestamp)
				} else {
					currentLocation := time.Now().Location()
					firstOfMonth := time.Date(year, time.Month(month), 1, 0, 0, 0, 0, currentLocation)
					dataTime = firstOfMonth.AddDate(0, 1, -1)
					sDataTime = dataTime.Format(utils.FormatDate)
					timestamp = dataTime.UnixNano() / 1e6
					timeStr = fmt.Sprintf("%d", timestamp)
				}
			} else {
				sDataTime = eDate
				dataTime, err = time.ParseInLocation(utils.FormatDate, eDate, time.Local)
				if err != nil {
					fmt.Println("time.Parse Err:" + eDate)
					return err
				}
				timestamp = dataTime.UnixNano() / 1e6
				timeStr = fmt.Sprintf("%d", timestamp)
			}
			sValue := itemValue
			if sValue != "" {
				saveValue := sValue

				if findItem, ok := existMap[eDate]; !ok {
					addSql += GetAddSql(edbInfoIdStr, edbCode, sDataTime, timeStr, saveValue)
					isAdd = true
				} else {
					if findItem != nil && utils.SubFloatToString(findItem.Value, 30) != sValue {
						err = ModifyEdbDataById(source, subSource, findItem.EdbDataId, sValue)
						if err != nil {
							return err
						}
					}
				}
			}

			// 下面代码主要目的是处理掉手动插入的数据判断
			{
				if realDataMaxDate.IsZero() || dataTime.After(realDataMaxDate) {
					realDataMaxDate = dataTime
				}
				if edbDataInsertConfigDate.IsZero() || dataTime.Equal(edbDataInsertConfigDate) {
					isFindConfigDateRealData = true
				}
			}

		}
	}

	// 处理手工数据补充的配置
	HandleConfigInsertEdbData(realDataMaxDate, edbDataInsertConfig, edbInfoId, source, subSource, existMap, isFindConfigDateRealData)

	if isAdd {
		addSql = strings.TrimRight(addSql, ",")
		//_, err = o.Raw(addSql).Exec()
		err = global.DEFAULT_DB.Exec(addSql).Error
		if err != nil {
			return err
		}
	}

	return
}
