package models

import (
	sql2 "database/sql"
	"eta/eta_index_lib/global"
	"eta/eta_index_lib/utils"
	"fmt"
	"gorm.io/gorm"
	"time"
)

const (
	CoalmineJsmIndexName     = `base_from_coalmine_jsm_index`
	CoalmineCompanyIndexName = `base_from_coalmine_company_index`
	CoalmineInlandIndexName  = `base_from_coalmine_inland_index`
	CoalmineFirmIndexName    = `base_from_coalmine_firm_index`
	CoalmineCoastalIndexName = `base_from_coalmine_coastal_index`
)

type BaseFromCoalmineMapping struct {
	BaseFromCoalmineMappingId int `gorm:"column:base_from_coalmine_mapping_id;primaryKey"`
	//BaseFromCoalmineMappingId int       `orm:"column(base_from_coalmine_mapping_id);pk"`
	IndexName  string    `description:"持买单量指标名称"`
	IndexCode  string    `description:"持买单量指标编码"`
	CreateTime time.Time `description:"时间"`
}

type BaseFromCoalmineJsmIndex struct {
	BaseFromCoalmineJsmIndexId int `gorm:"column:base_from_coalmine_jsm_index_id;primaryKey"`
	//BaseFromCoalmineJsmIndexId int       `orm:"column(base_from_coalmine_jsm_index_id);pk"`
	IndexName   string    `description:"持买单量指标名称"`
	IndexCode   string    `description:"持买单量指标编码"`
	Exchange    string    `description:"样本统计类别"`
	DealValue   string    `description:"成交量"`
	DataTime    string    `description:"数据日期"`
	Source      string    `description:"来源"`
	Province    string    `description:"省份"`
	Description string    `description:"描述"`
	Unit        string    `description:"单位"`
	Frequency   string    `description:"频率"`
	CreateTime  time.Time `description:"插入时间"`
	ModifyTime  time.Time `description:"修改时间"`
}

func (e *BaseFromCoalmineJsmIndex) AfterFind(db *gorm.DB) (err error) {
	e.DataTime = utils.GormDateStrToDateStr(e.DataTime)

	return
}

type BaseFromCoalmineCompanyIndex struct {
	BaseFromCoalmineCompanyIndexId int `gorm:"column:base_from_coalmine_company_index_id;primaryKey"`
	//BaseFromCoalmineCompanyIndexId int       `orm:"column(base_from_coalmine_company_index_id);pk"`
	IndexName  string    `description:"持买单量指标名称"`
	IndexCode  string    `description:"持买单量指标编码"`
	DealValue  string    `description:"成交量"`
	DataTime   string    `description:"数据日期"`
	Source     string    `description:"来源"`
	Province   string    `description:"省份"`
	City       string    `description:"城市"`
	GroupName  string    `description:"集团名称"`
	Unit       string    `description:"单位"`
	Frequency  string    `description:"频率"`
	CreateTime time.Time `description:"插入时间"`
	ModifyTime time.Time `description:"修改时间"`
}

func (e *BaseFromCoalmineCompanyIndex) AfterFind(db *gorm.DB) (err error) {
	e.DataTime = utils.GormDateStrToDateStr(e.DataTime)

	return
}

// 添加指标
func AddBaseFromCoalmineMapping(item *BaseFromCoalmineMapping) (lastId int64, err error) {
	//o := orm.NewOrm()
	//lastId, err = o.Insert(item)
	err = global.DEFAULT_DB.Create(&item).Error
	return
}

// 查询指标
func GetBaseFromCoalmineMapping() (items []*BaseFromCoalmineMapping, err error) {
	//o := orm.NewOrm()
	sql := `SELECT * FROM base_from_coalmine_mapping`
	//_, err = o.Raw(sql).QueryRows(&items)
	err = global.DEFAULT_DB.Raw(sql).Find(&items).Error
	return
}

// 查询数据
func GetBaseFromCoalmineIndex() (items []*BaseFromCoalmineJsmIndex, err error) {
	//o := orm.NewOrm()
	sql := `SELECT * FROM base_from_coalmine_jsm_index`
	err = global.DEFAULT_DB.Raw(sql).Find(&items).Error
	//_, err = o.Raw(sql).QueryRows(&items)
	return
}

func UpdateBaseFromCoalmineIndex(item *BaseFromCoalmineJsmIndex) (err error) {
	//o := orm.NewOrm()
	sql := `UPDATE base_from_coalmine_jsm_index SET deal_value=?  WHERE index_name=?  AND  data_time = ?`
	//_, err = o.Raw(sql, item.DealValue, item.IndexName, item.DataTime).Exec()
	err = global.DEFAULT_DB.Exec(sql, item.DealValue, item.IndexName, item.DataTime).Error
	return
}

// 添加数据
func AddBaseFromCoalmineIndex(item *BaseFromCoalmineJsmIndex) (lastId int64, err error) {
	//o := orm.NewOrm()
	//lastId, err = o.Insert(item)
	err = global.DEFAULT_DB.Create(&item).Error
	if err != nil {
		return
	}
	lastId = int64(item.BaseFromCoalmineJsmIndexId)
	return
}

// 添加公司指标
func AddBaseFromCoalmineCompanyIndex(item *BaseFromCoalmineCompanyIndex) (lastId int64, err error) {
	//o := orm.NewOrm()
	//lastId, err = o.Insert(item)
	err = global.DEFAULT_DB.Create(&item).Error
	if err != nil {
		return
	}
	lastId = int64(item.BaseFromCoalmineCompanyIndexId)
	return
}

// 查询公司指标
func GetBaseFromCoalmineCompanyIndex() (items []*BaseFromCoalmineCompanyIndex, err error) {
	//o := orm.NewOrm()
	sql := `SELECT * FROM base_from_coalmine_company_index`
	//_, err = o.Raw(sql).QueryRows(&items)
	err = global.DEFAULT_DB.Raw(sql).Find(&items).Error
	return
}

func UpdateBaseFromCoalmineCompanyIndex(item *BaseFromCoalmineCompanyIndex) (err error) {
	//o := orm.NewOrm()
	sql := `UPDATE base_from_coalmine_company_index SET deal_value=?  WHERE index_name=?  AND  data_time = ?`
	err = global.DEFAULT_DB.Exec(sql, item.DealValue, item.IndexName, item.DataTime).Error
	//_, err = o.Raw(sql, item.DealValue, item.IndexName, item.DataTime).Exec()
	return
}

func (m *BaseFromCoalmineMapping) GetMaxAndMinDateByIndexCode(indexCode, tableName string) (item *EdbInfoMaxAndMinInfo, err error) {
	if tableName == `` {
		return
	}
	//o := orm.NewOrm()
	sql := ` SELECT MIN(data_time) AS min_date,MAX(data_time) AS max_date,MIN(deal_value) AS min_value,MAX(deal_value) AS max_value FROM %s WHERE index_code=? `
	sql = fmt.Sprintf(sql, tableName)
	err = global.DEFAULT_DB.Raw(sql, indexCode).First(&item).Error
	//err = o.Raw(sql, indexCode).QueryRow(&item)
	if err != nil {
		return
	}

	// 获取最新值
	sql = ` SELECT deal_value AS latest_value FROM %s WHERE index_code=? ORDER BY data_time DESC LIMIT 1 `
	sql = fmt.Sprintf(sql, tableName)
	//err = o.Raw(sql, indexCode).QueryRow(&latest_value)
	var latestValueNull sql2.NullFloat64
	err = global.DEFAULT_DB.Raw(sql, indexCode).Scan(&latestValueNull).Error
	if err == nil && latestValueNull.Valid {
		item.LatestValue = latestValueNull.Float64
	}

	return
}

func (m *BaseFromCoalmineMapping) ModifyIndexMaxAndMinDate(indexCode string, item *EdbInfoMaxAndMinInfo) (err error) {
	//o := orm.NewOrm()
	sql := ` UPDATE base_from_coalmine_mapping SET start_date=?,end_date=?,latest_value=?,modify_time=NOW() WHERE index_code=? `
	err = global.DEFAULT_DB.Exec(sql, item.MinDate, item.MaxDate, item.LatestValue, indexCode).Error
	//_, err = o.Raw(sql, item.MinDate, item.MaxDate, item.LatestValue, indexCode).Exec()
	return
}

type CoalMineDataReq struct {
	SheetData []SheetData
}

type SheetData struct {
	Name     string
	Rows     []Row
	Cols     []*Col
	MaxRow   int
	MaxCol   int
	Hidden   bool
	Selected bool
}

type Row struct {
	Cells        []Cell
	Hidden       bool
	Height       float64
	OutlineLevel uint8
	isCustom     bool
}

type Col struct {
	Min          int
	Max          int
	Hidden       bool
	Width        float64
	Collapsed    bool
	OutlineLevel uint8
	numFmt       string
}

type Cell struct {
	Value string
}
