package models

import (
	sql2 "database/sql"
	"eta/eta_index_lib/global"
	"eta/eta_index_lib/utils"
	"fmt"
	"gorm.io/gorm"
	"strconv"
	"strings"
	"time"
)

// BaseFromUsdaFas 美国农业部
type BaseFromUsdaFas struct{}

type BaseFromUsdaFasData struct {
	BaseFromUsdaFasDataId int `gorm:"column:base_from_usda_fas_data_id;primaryKey"`
	//BaseFromUsdaFasDataId  int `orm:"column(base_from_usda_fas_data_id);pk"`
	BaseFromUsdaFasIndexId int
	IndexCode              string
	DataTime               string
	Value                  string
	CreateTime             time.Time
	ModifyTime             time.Time
	DataTimestamp          int64
}

func (e *BaseFromUsdaFasData) AfterFind(db *gorm.DB) (err error) {
	e.DataTime = utils.GormDateStrToDateStr(e.DataTime)

	return
}

func GetBaseFromUsdaFasDataByCondition(condition string, pars []interface{}) (list []*BaseFromUsdaFasData, err error) {
	//o := orm.NewOrm()
	sql := `SELECT * FROM base_from_usda_fas_data WHERE 1=1 `
	if condition != "" {
		sql += condition
	}
	//_, err = o.Raw(sql, pars).QueryRows(&list)
	err = global.DEFAULT_DB.Raw(sql, pars...).Find(&list).Error
	return
}

// Add 添加
func (obj BaseFromUsdaFas) Add(edbCode string) (err error) {
	//o := orm.NewOrm()

	var condition string
	var pars []interface{}
	if edbCode != "" {
		condition += " AND index_code=? "
		pars = append(pars, edbCode)
	}
	UsdaFasBaseDataAll, err := GetBaseFromUsdaFasDataByCondition(condition, pars)
	if err != nil && !utils.IsErrNoRow(err) {
		return
	}
	var isAdd bool
	addSql := ` INSERT INTO edb_data_usda_fas(edb_info_id,edb_code,data_time,value,create_time,modify_time,data_timestamp) values `
	existMap := make(map[string]string)
	for _, sv := range UsdaFasBaseDataAll {
		eDate := sv.DataTime
		dataTime, err := time.Parse(utils.FormatDate, eDate)
		if err != nil {
			return err
		}
		timestamp := dataTime.UnixNano() / 1e6
		timeStr := fmt.Sprintf("%d", timestamp)
		if _, ok := existMap[eDate]; !ok {
			addSql += GetAddSql("0", edbCode, eDate, timeStr, sv.Value)
			isAdd = true
		}
		existMap[eDate] = sv.Value
	}
	if isAdd {
		addSql = strings.TrimRight(addSql, ",")
		utils.FileLog.Info("addSql:" + addSql)
		//_,err=o.Raw(addSql).Exec()
		err = global.DEFAULT_DB.Exec(addSql).Error
		if err != nil {
			return err
		}
	}
	return
}

// Refresh 刷新涌益咨询指标数据
func (obj BaseFromUsdaFas) Refresh(edbInfoId int, edbCode, startDate string) (err error) {
	source := obj.GetSource()
	//o := orm.NewOrm()
	if err != nil {
		return
	}
	edbInfoIdStr := strconv.Itoa(edbInfoId)
	//计算数据
	var condition string
	var pars []interface{}

	if edbCode != "" {
		condition += " AND index_code=? "
		pars = append(pars, edbCode)
	}

	if startDate != "" {
		condition += " AND data_time>=? "
		pars = append(pars, startDate)
	}

	UsdaFasDataList, err := GetBaseFromUsdaFasDataByCondition(condition, pars)
	if err != nil {
		return
	}

	// 真实数据的最大日期  ， 插入规则配置的日期
	var realDataMaxDate, edbDataInsertConfigDate time.Time
	var edbDataInsertConfig *EdbDataInsertConfig
	var isFindConfigDateRealData bool //是否找到配置日期的实际数据的值
	{
		edbDataInsertConfig, err = GetEdbDataInsertConfigByEdbId(edbInfoId)
		if err != nil && !utils.IsErrNoRow(err) {
			return
		}
		if edbDataInsertConfig != nil {
			edbDataInsertConfigDate = edbDataInsertConfig.Date
		}
	}

	var existCondition string
	var existPars []interface{}

	existCondition += " AND edb_info_id=? "
	existPars = append(existPars, edbInfoId)
	if startDate != "" {
		existCondition += " AND data_time>=? "
		existPars = append(existPars, startDate)
	}

	existList, err := GetEdbDataByCondition(source, 0, existCondition, existPars)
	if err != nil {
		return err
	}
	existMap := make(map[string]*EdbInfoSearchData)
	for _, v := range existList {
		existMap[v.DataTime] = v
	}
	addSql := ` INSERT INTO edb_data_usda_fas(edb_info_id,edb_code,data_time,value,create_time,modify_time,data_timestamp) values `
	var isAdd bool
	for _, v := range UsdaFasDataList {
		item := v
		eDate := item.DataTime
		dataTime, err := time.ParseInLocation(utils.FormatDate, eDate, time.Local)
		if err != nil {
			return err
		}
		if findItem, ok := existMap[v.DataTime]; !ok {
			sValue := item.Value

			timestamp := dataTime.UnixNano() / 1e6
			timeStr := fmt.Sprintf("%d", timestamp)

			addSql += GetAddSql(edbInfoIdStr, edbCode, eDate, timeStr, sValue)
			isAdd = true
		} else {
			if findItem != nil && utils.SubFloatToString(findItem.Value, 30) != item.Value {
				err = ModifyEdbDataById(source, 0, findItem.EdbDataId, item.Value)
				if err != nil {
					return err
				}
			}
		}

		// 下面代码主要目的是处理掉手动插入的数据判断
		{
			if realDataMaxDate.IsZero() || dataTime.After(realDataMaxDate) {
				realDataMaxDate = dataTime
			}
			if edbDataInsertConfigDate.IsZero() || dataTime.Equal(edbDataInsertConfigDate) {
				isFindConfigDateRealData = true
			}
		}
	}

	// 处理手工数据补充的配置
	HandleConfigInsertEdbData(realDataMaxDate, edbDataInsertConfig, edbInfoId, source, 0, existMap, isFindConfigDateRealData)

	if isAdd {
		addSql = strings.TrimRight(addSql, ",")
		//_,err = o.Raw(assSql).Exec()
		err = global.DEFAULT_DB.Exec(addSql).Error
		if err != nil {
			fmt.Println("RefreshEdbDataFromBaiinfo add Err", err.Error())
			return
		}
	}
	return
}

// GetSource 获取来源编码id
func (obj BaseFromUsdaFas) GetSource() int {
	return utils.DATA_SOURCE_USDA_FAS
}

// GetSourceName 获取来源名称
func (obj BaseFromUsdaFas) GetSourceName() string {
	return utils.DATA_SOURCE_NAME_USDA_FAS
}

type BaseFromUsdaFasIndex struct {
	BaseFromUsdaFasIndexId int64 `gorm:"column:base_from_usda_fas_index_id;primaryKey"`
	//BaseFromUsdaFasIndexId int64 `orm:"column(base_from_usda_fas_index_id);pk"`
	IndexCode      string
	IndexName      string
	Frequency      string
	Unit           string
	StartDate      string
	EndDate        string
	ClassifyId     int64
	Sort           int
	BaseFileName   string
	RenameFileName string
	TerminalCode   string
	Country        string `description:"国家"`
	Commodity      string `description:"属性"`
	CreateTime     time.Time
	ModifyTime     time.Time
}

func (e *BaseFromUsdaFasIndex) AfterFind(db *gorm.DB) (err error) {
	e.StartDate = utils.GormDateStrToDateStr(e.StartDate)
	e.EndDate = utils.GormDateStrToDateStr(e.EndDate)

	return
}

type BaseFromUsdaFasIndexList struct {
	BaseFromUsdaFasIndexId int64 `gorm:"column:base_from_usda_fas_index_id;primaryKey"`
	//BaseFromUsdaFasIndexId int64 `orm:"column(base_from_usda_fas_index_id);pk"`
	IndexCode    string
	IndexName    string
	Frequency    string
	Unit         string
	Sort         int
	ClassifyId   int64
	StartDate    string
	EndDate      string
	TerminalCode string
	CreateTime   string
	ModifyTime   string
}

// AfterFind 在该模型上设置钩子函数，把日期转成正确的string，所以查询函数只能用Find函数，First或者Scan是不会触发该函数的来获取数据
func (m *BaseFromUsdaFasIndexList) AfterFind(db *gorm.DB) (err error) {
	m.StartDate = utils.GormDateStrToDateStr(m.StartDate)
	m.EndDate = utils.GormDateStrToDateStr(m.EndDate)

	return
}

func (y *BaseFromUsdaFasData) GetByIndexCode(indexCode string) (list []*BaseFromUsdaFasData, err error) {
	//o := orm.NewOrm()
	sql := ` SELECT * FROM base_from_usda_fas_data WHERE index_code=? `
	//_, err = o.Raw(sql, indexCode).QueryRows(&list)
	err = global.DEFAULT_DB.Raw(sql, indexCode).Find(&list).Error
	return
}

func (y *BaseFromUsdaFasData) AddMulti(item []*BaseFromUsdaFasData) (err error) {
	//o := orm.NewOrm()
	//_, err = o.InsertMulti(1, item)
	err = global.DEFAULT_DB.CreateInBatches(&item, 1).Error
	return
}

// Update 修改
func (y *BaseFromUsdaFasData) Update(updateCols []string) (err error) {
	//o := orm.NewOrm()
	//_, err = o.Update(y, updateCols...)
	err = global.DEFAULT_DB.Model(&y).Select(updateCols).Updates(&y).Error
	return
}

// HandleUsdaFasExcelData 涌益咨询的excel数据
type HandleUsdaFasExcelData struct {
	ClassifyName       string `description:"指标目录"`
	ParentClassifyName string `description:"父级指标目录"`
	ClassifySort       int    `description:"指标目录排序号"`
	IndexName          string `description:"指标名称"`
	IndexCode          string `description:"指标编码"`
	Unit               string `description:"单位"`
	Sort               int    `description:"排序号"`
	Frequency          string `description:"频度"`
	Country            string `description:"国家"`
	Commodity          string `description:"属性"`
	ExcelDataMap       map[string]string
}

type HandleUsdaFasExcelDataReq struct {
	List         []*HandleUsdaFasExcelData
	TerminalCode string `description:"编码"`
}

func (y *BaseFromUsdaFasData) GetMaxAndMinDateByIndexCode(indexCode string) (item *EdbInfoMaxAndMinInfo, err error) {
	//o := orm.NewOrm()
	sql := ` SELECT MIN(data_time) AS min_date,MAX(data_time) AS max_date,MIN(value) AS min_value,MAX(value) AS max_value FROM base_from_usda_fas_data WHERE index_code=? `
	//err = o.Raw(sql, indexCode).QueryRow(&item)
	err = global.DEFAULT_DB.Raw(sql, indexCode).First(&item).Error
	sql = ` SELECT value AS latest_value FROM %s WHERE index_code=? ORDER BY data_time DESC LIMIT 1 `
	sql = fmt.Sprintf(sql, "base_from_usda_fas_data")
	//err = o.Raw(sql, indexCode).QueryRow(&latest_value)
	var latestValueNull sql2.NullFloat64
	err = global.DEFAULT_DB.Raw(sql, indexCode).Scan(&latestValueNull).Error
	if err == nil && latestValueNull.Valid {
		item.LatestValue = latestValueNull.Float64
	}

	return
}

func (y *BaseFromUsdaFasIndex) ModifyIndexMaxAndMinDate(indexCode string, item *EdbInfoMaxAndMinInfo) (err error) {
	//o := orm.NewOrm()
	sql := ` UPDATE base_from_usda_fas_index SET start_date=?,end_date=?, end_value=?, modify_time=NOW() WHERE index_code=? `
	//_, err = o.Raw(sql, item.MinDate, item.MaxDate, item.LatestValue, indexCode).Exec()
	err = global.DEFAULT_DB.Exec(sql, item.MinDate, item.MaxDate, item.LatestValue, indexCode).Error
	return
}

func (y *BaseFromUsdaFasIndex) GetByIndexCode(indexCode string) (item *BaseFromUsdaFasIndex, err error) {
	//o := orm.NewOrm()
	sql := ` SELECT * FROM base_from_usda_fas_index WHERE index_code=? `
	//err = o.Raw(sql, indexCode).QueryRow(&item)
	err = global.DEFAULT_DB.Raw(sql, indexCode).First(&item).Error
	return
}

func (y *BaseFromUsdaFasIndex) Add() (lastId int64, err error) {
	//o := orm.NewOrm()
	//lastId, err = o.Insert(y)
	err = global.DEFAULT_DB.Create(&y).Error
	if err != nil {
		return
	}
	lastId = y.BaseFromUsdaFasIndexId
	return
}

// Update 修改
func (y *BaseFromUsdaFasIndex) Update(updateCols []string) (err error) {
	//o := orm.NewOrm()
	//_, err = o.Update(y, updateCols...)
	err = global.DEFAULT_DB.Model(&y).Select(updateCols).Updates(&y).Error
	return
}

// BaseFromUsdaFasClassify UsdaFas原始数据分类表
type BaseFromUsdaFasClassify struct {
	ClassifyId int64 `gorm:"column:classify_id;primaryKey"`
	//ClassifyId      int64     `orm:"column(classify_id);pk"`
	ClassifyName    string    `description:"分类名称"`
	ClassifyNameEn  string    `description:"分类名称"`
	ParentId        int       `description:"父级id"`
	SysUserId       int       `description:"创建人id"`
	SysUserRealName string    `description:"创建人姓名"`
	Level           int       `description:"层级"`
	Sort            int       `description:"排序字段，越小越靠前，默认值：10"`
	ModifyTime      time.Time `description:"修改时间"`
	CreateTime      time.Time `description:"创建时间"`
}

func (y *BaseFromUsdaFasClassify) Add() (lastId int64, err error) {
	//o := orm.NewOrm()
	//lastId, err = o.Insert(y)
	err = global.DEFAULT_DB.Create(&y).Error
	if err != nil {
		return
	}
	lastId = int64(y.ClassifyId)
	return
}

// Update 修改
func (y *BaseFromUsdaFasClassify) Update(updateCols []string) (err error) {
	//o := orm.NewOrm()
	//_, err = o.Update(y, updateCols...)
	err = global.DEFAULT_DB.Model(&y).Select(updateCols).Updates(&y).Error
	return
}

func (y *BaseFromUsdaFasClassify) GetByClassifyName(classifyName string) (item *BaseFromUsdaFasClassify, err error) {
	//o := orm.NewOrm()
	sql := ` SELECT * FROM base_from_usda_fas_classify WHERE classify_name=? `
	//err = o.Raw(sql, classifyName).QueryRow(&item)
	err = global.DEFAULT_DB.Raw(sql, classifyName).First(&item).Error
	return
}

func (y *BaseFromUsdaFasClassify) GetParentClassify() (items []*BaseFromUsdaFasClassify, err error) {
	//o := orm.NewOrm()
	sql := ` SELECT * FROM base_from_usda_fas_classify WHERE parent_id=0 `
	//_, err = o.Raw(sql).QueryRows(&items)
	err = global.DEFAULT_DB.Raw(sql).Find(&items).Error
	return
}
