// @Author gmy 2024/9/14 16:13:00
package models

import (
	"eta/eta_index_lib/global"
	"eta/eta_index_lib/utils"
	"fmt"
	"strconv"
	"strings"
	"time"
)

type EdbDataLy struct {
	EdbDataId     int     `gorm:"column:edb_data_id;type:int(10) UNSIGNED;primaryKey;not null;"`
	CreateTime    string  `gorm:"column:create_time;type:datetime;comment:创建时间;not null;default:CURRENT_TIMESTAMP;"`  // 创建时间
	ModifyTime    string  `gorm:"column:modify_time;type:datetime;comment:修改时间;not null;default:CURRENT_TIMESTAMP;"`  // 修改时间
	EdbInfoId     int     `gorm:"column:edb_info_id;type:int(10) UNSIGNED;comment:指标id;not null;"`                    // 指标id
	EdbCode       string  `gorm:"column:edb_code;type:varchar(128);comment:指标编码;not null;"`                           // 指标编码
	DataTime      string  `gorm:"column:data_time;type:date;comment:数据日期;default:NULL;"`                              // 数据日期
	Value         float64 `gorm:"column:value;type:double;comment:数据值;default:NULL;"`                                 // 数据值
	DataTimestamp uint64  `gorm:"column:data_timestamp;type:bigint(20) UNSIGNED;comment:数据日期时间戳;not null;default:0;"` // 数据日期时间戳
}

// ConvertTimeStr
// @Description: 转成需要输出的格式
// @receiver m
func (m *EdbDataLy) ConvertTimeStr() {
	m.DataTime = utils.GormDateStrToDateStr(m.DataTime)
	m.CreateTime = utils.GormDateStrToDateTimeStr(m.DataTime)
	m.CreateTime = utils.GormDateStrToDateTimeStr(m.DataTime)

	return
}

func GetLyEdbDataByIndexCodeAndDataTime(indexCode string, dataTime string) (items []EdbDataLy, err error) {
	sql := `SELECT * FROM edb_data_ly WHERE index_code=? AND data_time like ?`
	err = global.DEFAULT_DB.Raw(sql, indexCode, dataTime+"%").Find(&items).Error
	if utils.IsErrNoRow(err) {
		return nil, nil
	}
	for _, v := range items {
		v.ConvertTimeStr()
	}

	return
}

func GetLyEdbDataByIndexCodeAndExactDataTime(indexCode string, dataTime string) (items []EdbDataLy, err error) {
	sql := `SELECT * FROM edb_data_ly WHERE edb_code=? AND data_time=?`
	err = global.DEFAULT_DB.Raw(sql, indexCode, dataTime).Find(&items).Error
	if utils.IsErrNoRow(err) {
		return nil, nil
	}
	for _, v := range items {
		v.ConvertTimeStr()
	}

	return
}

// UpdateLyEdbDataById 更新指标库数据 须根据指标编码和日期更新 仅适合月度数据
func UpdateLyEdbDataById(id int, value float64) (err error) {
	sql := `UPDATE edb_data_ly SET value=? WHERE edb_data_id=?`
	err = global.DEFAULT_DB.Exec(sql, value, id).Error

	return
}

// 新增指标库数据
func AddLyEdbDataList(items []EdbDataLy) (err error) {
	err = global.DEFAULT_DB.CreateInBatches(items, utils.MultiAddNum).Error

	return
}

// AddEdbDataFromLy 新增指标数据
func AddEdbDataFromLy(edbCode string) (err error) {
	o := global.DEFAULT_DB

	var condition string
	var pars []interface{}

	if edbCode != "" {
		condition += " AND index_code = ? "
		pars = append(pars, edbCode)
	}

	dataAll, err := GetBaseFromLyDataByIndexCode(condition, pars)
	if err != nil {
		return err
	}
	dataLen := len(dataAll)

	existMap := make(map[string]string)
	if dataLen > 0 {
		var isAdd bool
		addSql := ` INSERT INTO edb_data_ly (edb_info_id,edb_code,data_time,value,create_time,modify_time,data_timestamp) values `
		for i := 0; i < dataLen; i++ {
			item := dataAll[i]
			eDate := item.DataTime
			sValue := utils.SubFloatToString(item.Value, 4)
			if sValue != "" {
				if _, ok := existMap[eDate]; !ok {
					dataTime, err := time.ParseInLocation(utils.FormatDate, eDate, time.Local)
					if err != nil {
						return err
					}
					timestamp := dataTime.UnixNano() / 1e6
					timeStr := fmt.Sprintf("%d", timestamp)
					addSql += GetAddSql("0", edbCode, eDate, timeStr, sValue)
					isAdd = true
				}
			}
			existMap[eDate] = eDate
		}
		if isAdd {
			addSql = strings.TrimRight(addSql, ",")
			utils.FileLog.Info("addSql:" + addSql)
			err = o.Exec(addSql).Error
			if err != nil {
				return err
			}
		}
	}
	return
}

func RefreshEdbDataFromLy(edbInfoId int, edbCode, startDate string) (err error) {
	source := utils.DATA_SOURCE_LY
	subSource := utils.DATA_SUB_SOURCE_EDB

	o := global.DEFAULT_DB
	if err != nil {
		return
	}
	edbInfoIdStr := strconv.Itoa(edbInfoId)
	//计算数据
	var condition string
	var pars []interface{}

	if edbCode != "" {
		condition += " AND index_code=? "
		pars = append(pars, edbCode)
	}

	if startDate != "" {
		condition += " AND data_time>=? "
		pars = append(pars, startDate)
	}

	dataList, err := GetBaseFromLyDataByIndexCode(condition, pars)
	if err != nil {
		return
	}

	// 真实数据的最大日期  ， 插入规则配置的日期
	var realDataMaxDate, edbDataInsertConfigDate time.Time
	var edbDataInsertConfig *EdbDataInsertConfig
	var isFindConfigDateRealData bool //是否找到配置日期的实际数据的值
	{
		edbDataInsertConfig, err = GetEdbDataInsertConfigByEdbId(edbInfoId)
		if err != nil && !utils.IsErrNoRow(err) {
			return
		}
		if edbDataInsertConfig != nil {
			edbDataInsertConfigDate = edbDataInsertConfig.Date
		}
	}

	var existCondition string
	var existPars []interface{}

	existCondition += " AND edb_info_id=? "
	existPars = append(existPars, edbInfoId)
	if startDate != "" {
		existCondition += " AND data_time>=? "
		existPars = append(existPars, startDate)
	}
	//获取指标所有数据
	existList, err := GetEdbDataByCondition(source, subSource, existCondition, existPars)
	if err != nil {
		return err
	}
	existMap := make(map[string]*EdbInfoSearchData)
	for _, v := range existList {
		existMap[v.DataTime] = v
	}

	addSql := ` INSERT INTO ebd_data_ly(edb_info_id,edb_code,data_time,value,create_time,modify_time,data_timestamp) values `
	var isAdd bool
	addMap := make(map[string]string)
	for _, v := range dataList {
		item := v
		eDate := item.DataTime
		sValue := utils.SubFloatToString(item.Value, 4)

		dataTime, err := time.ParseInLocation(utils.FormatDate, eDate, time.Local)
		if err != nil {
			return err
		}
		if findItem, ok := existMap[v.DataTime]; !ok {
			if sValue != "" {
				timestamp := dataTime.UnixNano() / 1e6
				timeStr := fmt.Sprintf("%d", timestamp)
				saveValue := sValue

				if _, addOk := addMap[eDate]; !addOk {
					addSql += GetAddSql(edbInfoIdStr, edbCode, eDate, timeStr, saveValue)
					isAdd = true
				}
			}
		} else {
			if findItem != nil && utils.SubFloatToString(findItem.Value, 4) != sValue {
				err = ModifyEdbDataById(source, subSource, findItem.EdbDataId, sValue)
				if err != nil {
					return err
				}
			}
		}
		addMap[v.DataTime] = v.DataTime

		// 下面代码主要目的是处理掉手动插入的数据判断
		{
			if realDataMaxDate.IsZero() || dataTime.After(realDataMaxDate) {
				realDataMaxDate = dataTime
			}
			if edbDataInsertConfigDate.IsZero() || dataTime.Equal(edbDataInsertConfigDate) {
				isFindConfigDateRealData = true
			}
		}
	}

	// 处理手工数据补充的配置
	HandleConfigInsertEdbData(realDataMaxDate, edbDataInsertConfig, edbInfoId, source, subSource, existMap, isFindConfigDateRealData)

	if isAdd {
		addSql = strings.TrimRight(addSql, ",")
		err = o.Exec(addSql).Error
		if err != nil {
			return err
		}
	}
	return
}
