package models

import (
	"eta/eta_index_lib/global"
	"eta/eta_index_lib/utils"
	"fmt"
	"strings"
	"time"
)

// EdbDataThsHf 同花顺高频指标数据
type EdbDataThsHf struct {
	EdbDataId     int       `gorm:"column:edb_data_id;type:int(10) UNSIGNED;primaryKey;not null;"`
	EdbInfoId     int       `gorm:"column:edb_info_id;type:int(10) UNSIGNED;comment:指标ID;not null;default:0;"` // 指标ID
	EdbCode       string    `gorm:"column:edb_code;type:varchar(255);comment:指标编码;not null;"`                  // 指标编码
	DataTime      time.Time `gorm:"column:data_time;type:date;comment:数据日期;default:NULL;"`                     // 数据日期
	Value         float64   `gorm:"column:value;type:double;comment:数据值;default:NULL;"`                        // 数据值
	CreateTime    time.Time `gorm:"column:create_time;type:datetime;comment:创建时间;default:NULL;"`               // 创建时间
	ModifyTime    time.Time `gorm:"column:modify_time;type:datetime;comment:修改时间;default:NULL;"`               // 修改时间
	DataTimestamp int64     `gorm:"column:data_timestamp;type:bigint(20) UNSIGNED;comment:数据日期时间戳;default:0;"` // 数据日期时间戳
}

func (m *EdbDataThsHf) TableName() string {
	return "edb_data_ths_hf"
}

type EdbDataThsHfCols struct {
	PrimaryId     string
	EdbInfoId     string
	EdbCode       string
	DataTime      string
	Value         string
	CreateTime    string
	ModifyTime    string
	DataTimestamp string
}

func (m *EdbDataThsHf) Cols() EdbDataThsHfCols {
	return EdbDataThsHfCols{
		PrimaryId:     "edb_data_id",
		EdbInfoId:     "edb_info_id",
		EdbCode:       "edb_code",
		DataTime:      "data_time",
		Value:         "value",
		CreateTime:    "create_time",
		ModifyTime:    "modify_time",
		DataTimestamp: "data_timestamp",
	}
}

func (m *EdbDataThsHf) Create() (err error) {
	err = global.DEFAULT_DB.Create(m).Error
	if err != nil {
		return
	}

	return
}

func (m *EdbDataThsHf) CreateMulti(items []*EdbDataThsHf) (err error) {
	if len(items) == 0 {
		return
	}
	err = global.DEFAULT_DB.CreateInBatches(items, utils.MultiAddNum).Error

	return
}

func (m *EdbDataThsHf) Update(cols []string) (err error) {
	err = global.DEFAULT_DB.Select(cols).Updates(m).Error

	return
}

func (m *EdbDataThsHf) Remove() (err error) {
	sql := fmt.Sprintf(`DELETE FROM %s WHERE %s = ? LIMIT 1`, m.TableName(), m.Cols().PrimaryId)
	err = global.DEFAULT_DB.Exec(sql, m.EdbDataId).Error

	return
}

func (m *EdbDataThsHf) MultiRemove(ids []int) (err error) {
	if len(ids) == 0 {
		return
	}
	sql := fmt.Sprintf(`DELETE FROM %s WHERE %s IN (%s)`, m.TableName(), m.Cols().PrimaryId, utils.GetOrmInReplace(len(ids)))
	err = global.DEFAULT_DB.Exec(sql, ids).Error

	return
}

func (m *EdbDataThsHf) RemoveByCondition(condition string, pars []interface{}) (err error) {
	if condition == "" {
		return
	}
	sql := fmt.Sprintf(`DELETE FROM %s WHERE %s`, m.TableName(), condition)
	err = global.DEFAULT_DB.Exec(sql, pars).Error

	return
}

func (m *EdbDataThsHf) GetItemById(id int) (item *EdbDataThsHf, err error) {
	sql := fmt.Sprintf(`SELECT * FROM %s WHERE %s = ? LIMIT 1`, m.TableName(), m.Cols().PrimaryId)
	err = global.DEFAULT_DB.Raw(sql, id).First(&item).Error

	return
}

func (m *EdbDataThsHf) GetItemByCondition(condition string, pars []interface{}, orderRule string) (item *EdbDataThsHf, err error) {
	order := ``
	if orderRule != "" {
		order = ` ORDER BY ` + orderRule
	}
	sql := fmt.Sprintf(`SELECT * FROM %s WHERE 1=1 %s %s LIMIT 1`, m.TableName(), condition, order)
	err = global.DEFAULT_DB.Raw(sql, pars...).First(&item).Error

	return
}

func (m *EdbDataThsHf) GetCountByCondition(condition string, pars []interface{}) (count int, err error) {
	sql := fmt.Sprintf(`SELECT COUNT(1) FROM %s WHERE 1=1 %s`, m.TableName(), condition)
	err = global.DEFAULT_DB.Raw(sql, pars...).Scan(&count).Error

	return
}

func (m *EdbDataThsHf) GetItemsByCondition(condition string, pars []interface{}, fieldArr []string, orderRule string) (items []*EdbDataThsHf, err error) {
	fields := strings.Join(fieldArr, ",")
	if len(fieldArr) == 0 {
		fields = `*`
	}
	order := fmt.Sprintf(`ORDER BY %s DESC`, m.Cols().CreateTime)
	if orderRule != "" {
		order = ` ORDER BY ` + orderRule
	}
	sql := fmt.Sprintf(`SELECT %s FROM %s WHERE 1=1 %s %s`, fields, m.TableName(), condition, order)
	err = global.DEFAULT_DB.Raw(sql, pars...).Find(&items).Error

	return
}

func (m *EdbDataThsHf) GetPageItemsByCondition(condition string, pars []interface{}, fieldArr []string, orderRule string, startSize, pageSize int) (items []*EdbDataThsHf, err error) {
	fields := strings.Join(fieldArr, ",")
	if len(fieldArr) == 0 {
		fields = `*`
	}
	order := fmt.Sprintf(`ORDER BY %s DESC`, m.Cols().CreateTime)
	if orderRule != "" {
		order = ` ORDER BY ` + orderRule
	}
	sql := fmt.Sprintf(`SELECT %s FROM %s WHERE 1=1 %s %s LIMIT ?,?`, fields, m.TableName(), condition, order)
	pars = append(pars, startSize, pageSize)
	err = global.DEFAULT_DB.Raw(sql, pars...).Find(&items).Error

	return
}

// MultiUpdateValue 批量更新数据值
func (m *EdbDataThsHf) MultiUpdateValue(items []*EdbDataThsHf) (err error) {
	if len(items) == 0 {
		return
	}

	o := global.DEFAULT_DB.Begin()
	defer func() {
		if err != nil {
			o.Rollback()
		} else {
			o.Commit()
		}
	}()

	sql := fmt.Sprintf("UPDATE %s SET %s = ?, %s = NOW() WHERE %s = ?", m.TableName(), m.Cols().Value, m.Cols().ModifyTime, m.Cols().PrimaryId)
	for _, v := range items {
		err = o.Exec(sql, v.Value, v.EdbDataId).Error
		if err != nil {
			return
		}
	}
	return
}
